@@ -96,7 +96,7 @@ html_document_base <- function(theme = NULL,
 
     # Process css files as Pandoc argument if not already been processed by bslib
     for (f in css) {
-      if (grepl("\\.s[ac]ss$", f)) {
+      if (needs_sass(f)) {
         if (!xfun::loadable("sass")) {
           stop2("Using `.sass` or `.scss` file in `css` argument requires the sass package.")
         }
@@ -237,7 +237,10 @@ html_document_base <- function(theme = NULL,
     knitr = NULL,
     pandoc = pandoc_options(
       to = "html", from = NULL, args = args,
-      lua_filters = pkg_file_lua(c("pagebreak.lua", "latex-div.lua"))
+      lua_filters = pkg_file_lua(c(
+        "pagebreak.lua", "latex-div.lua",
+        if (pandoc_available("3.2.1")) "table-classes.lua"
+      ))
     ),
     keep_md = FALSE,
     clean_supporting = FALSE,
@@ -251,7 +254,12 @@ html_document_base <- function(theme = NULL,
 }
 
 extract_preserve_chunks <- function(input_file, extract = extractPreserveChunks) {
-  input_str <- read_utf8(input_file)
+  # Don't try to modify the input file if it's not .md, otherwise the original
+  # input could be corrupted (#2534). In theory, preserved chunks should only
+  # exist in the intermediate .md file from knit(). If the .md file is not
+  # intermediate but original, this processing should be harmless.
+  if (!xfun::file_ext(input_file) %in% c('md', 'markdown')) return()
+  input_str <- one_string(read_utf8(input_file))
   preserve <- extract(input_str)
   if (!identical(preserve$value, input_str)) write_utf8(preserve$value, input_file)
   preserve$chunks
