@@ -161,11 +161,13 @@ params_get_input <- function(param) {
   input <- param$input
   if (is.null(input)) {
     if (!is.null(param$choices)) {
-      ## radio buttons for a small number of choices, select otherwise.
-      if (length(param$choices) <= 4) {
-        input <- "radio"
-      } else {
+      ## select for a large number of choices and multiple choices.
+      if (isTRUE(param$multiple)) {
         input <- "select"
+      } else if (length(param$choices) > 4) {
+        input <- "select"
+      } else {
+        input <- "radio"
       }
     } else {
       ## Not choices. Look at the value type to find what input control we
@@ -219,8 +221,7 @@ params_configurable <- function(param) {
   }
   # Some inputs (like selectInput) support the selection of
   # multiple entries through a "multiple" argument.
-  multiple_ok <- (!is.null(param$multiple) && param$multiple)
-  if (multiple_ok) {
+  if (isTRUE(param$multiple)) {
     return(TRUE)
   }
   # sliderInput supports either one or two-value inputs.
@@ -245,8 +246,7 @@ params_namedList <- function() {
 #' @param params A named list of optional parameter overrides used in place of the document defaults.
 #' @param shiny_args Additional arguments to \code{\link[shiny:runApp]{runApp}}.
 #' @param save_caption Caption to use use for button that saves/confirms parameters.
-#' @param encoding The encoding of the input file; see \code{\link{file}}.
-#'
+#' @inheritParams render
 #' @return named list with overridden parameter names and value.
 #'
 #' @export
@@ -255,13 +255,13 @@ knit_params_ask <- function(file = NULL,
                             params = NULL,
                             shiny_args = NULL,
                             save_caption = "Save",
-                            encoding = getOption("encoding")) {
+                            encoding = "UTF-8") {
 
   if (is.null(input_lines)) {
     if (is.null(file)) {
       stop("knit_params_ask must have a non-NULL file or input_lines parameter")
     }
-    input_lines <- read_utf8(file, encoding)
+    input_lines <- read_utf8(file)
   }
 
   knit_params <- knitr::knit_params(input_lines)
