@@ -1,22 +1,30 @@
 
 # Create a shiny app object from an Rmd w/ runtime: shiny_prerendered
-shiny_prerendered_app <- function(input_rmd, encoding, render_args) {
+shiny_prerendered_app <- function(input_rmd, render_args) {
 
   # get rendered html and capture dependencies
-  html <- shiny_prerendered_html(input_rmd, encoding, render_args)
-  deps <- attr(html, "html_dependencies")
+  html <- shiny_prerendered_html(input_rmd, render_args)
+  deps <- htmltools::htmlDependencies(html)
 
   # create the server environment
   server_envir = new.env(parent = globalenv())
 
-  # extract the server-start context
+  # read the html
   html_lines <- strsplit(html, "\n", fixed = TRUE)[[1]]
+
+  # read and remove server-extras (used for both embedded and server.R use cases)
+  server_extras <- shiny_prerendered_extract_context(html_lines, "server-extras")
+  if (length(server_extras) > 0) {
+    html_lines <- shiny_prerendered_remove_contexts(html_lines, "server-extras")
+  }
+
+  # extract the server-start context
   server_start_context <- shiny_prerendered_extract_context(html_lines,
                                                             "server-start")
 
   # extract the code used for server startup (this encompasses both the
   # context="server-start" code and the context="data" code). This can be
-  # retreived later via the shiny_prerendered_server_start_code function. The
+  # retrieved later via the shiny_prerendered_server_start_code function. The
   # purpose of this is for appliations which want to run user in other
   # processes while still duplicating the setup context (e.g. tutorials).
   server_start_code <- one_string(c(server_start_context,
@@ -28,40 +36,66 @@ shiny_prerendered_app <- function(input_rmd, encoding, render_args) {
     assign(".shiny_prerendered_server_start_code", server_start_code, envir = server_envir)
 
     # execute the startup code (server_start_context + context="data" loading)
-    eval(parse(text = server_start_context), envir = server_envir)
+    eval(xfun::parse_only(server_start_context), envir = server_envir)
     shiny_prerendered_data_load(input_rmd, server_envir)
 
-    # lock the environment to prevent inadvertant assignments
+    # lock the environment to prevent inadvertent assignments
     lockEnvironment(server_envir)
   }
 
   # extract the server context
-  .server_context <- shiny_prerendered_extract_context(html_lines, "server")
+  .server_context <- c(server_extras,
+                       shiny_prerendered_extract_context(html_lines, "server"))
   server_envir$.server_context <- .server_context
   server <- function(input, output, session) {
-    eval(parse(text = .server_context))
+    eval(xfun::parse_only(.server_context))
   }
   environment(server) <- new.env(parent = server_envir)
 
   # remove server code before serving
   server_contexts <-  c("server-start", "data", "server")
-  html_lines <- shiny_prerendered_remove_contexts(html_lines, server_contexts)
-  html <- HTML(one_string(html_lines))
-  html <- htmltools::attachDependencies(html, deps)
+  redacted_html_lines <- shiny_prerendered_remove_contexts(html_lines, server_contexts)
+
+  # if there were server contexts then update html w/ removed contexts
+  if (length(redacted_html_lines) < length(html_lines)) {
+    html <- HTML(one_string(redacted_html_lines))
+  }
+  # if there were not server contexts then this may be a ui-only rmd,
+  # check for a server.R
+  else if (file.exists(file.path(dirname(input_rmd), "server.R"))) {
+    # source global.R onStart
+    onStart <- function() {
+      global_r <- file.path.ci(dirname(input_rmd), "global.R")
+      if (file.exists(global_r)) {
+        source(global_r, local = FALSE)
+      }
+    }
+    # server function from server.R
+    server_r <- file.path(dirname(input_rmd), "server.R")
+    server_r_env = new.env(parent = globalenv())
+    if (length(server_extras) > 0) {
+      eval(xfun::parse_only(server_extras), envir = server_r_env)
+    }
+    server <- source(server_r, local = server_r_env)$value
+  } else {
+    stop("No server contexts or server.R available for ", input_rmd)
+  }
+
+  html_ui <- shiny_prerendered_ui(html, deps)
 
   # create shiny app
   shiny::shinyApp(
-    ui = function(req) html,
+    ui = function(req) html_ui,
     server = server,
     onStart = onStart,
-    uiPattern = "^/$|^(/.*\\.[Rr][Mm][Dd])$"
+    uiPattern = "^/$|^(/.*\\.[Rrq][Mm][Dd])$"
   )
 }
 
 
 # Generate the html for a runtime: shiny_prerendered Rmd (attempts to use
 # an existing rendering of the html if it's still valid)
-shiny_prerendered_html <- function(input_rmd, encoding, render_args) {
+shiny_prerendered_html <- function(input_rmd, render_args) {
 
   # determine the path to the rendered_html
   output_file <- render_args$output_file
@@ -78,15 +112,17 @@ shiny_prerendered_html <- function(input_rmd, encoding, render_args) {
     input_rmd,
     rendered_html,
     output_dir,
-    encoding,
     prerender_option
   )
 
   # prerender if necessary
   if (prerender) {
+    args <- merge_lists(list(input = input_rmd), render_args)
+    # force prerender to execute in separate environment to ensure that
+    # running w/ prerender step is equivalent to running the prerendered app
+    args$envir <- new.env(parent = args$envir %||% globalenv())
 
     # execute the render
-    args <- merge_lists(list(input = input_rmd, encoding = encoding), render_args)
     rendered_html <- do.call(render, args)
   }
 
@@ -95,14 +131,21 @@ shiny_prerendered_html <- function(input_rmd, encoding, render_args) {
   }
 
   # normalize paths
-  rendered_html <- normalize_path(rendered_html, winslash = "/")
+  rendered_html <- normalize_path(rendered_html)
   output_dir <- dirname(rendered_html)
 
-  # add some resource paths
-  add_resource_path <- function(path) {
-    if (dir_exists(path))
-      shiny::addResourcePath(basename(path), path)
+  add_resource_path <- function(path, prefix = basename(path), temporary = TRUE) {
+    if (dir_exists(path)) {
+      shiny::addResourcePath(prefix, path)
+      # Remove resource paths so they don't clash with 'down-stream' resources
+      removeResourcePath <- if (temporary)
+        try(getFromNamespace("removeResourcePath", "shiny"), silent = TRUE)
+      if (is.function(removeResourcePath)) {
+        shiny::onStop(function() { removeResourcePath(prefix) }, NULL)
+      }
+    }
   }
+
   files_dir <- knitr_files_dir(rendered_html)
   add_resource_path(files_dir)
   add_resource_path(file.path(output_dir,"css"))
@@ -112,8 +155,7 @@ shiny_prerendered_html <- function(input_rmd, encoding, render_args) {
 
   # extract dependencies from html
   html_lines <- read_utf8(rendered_html)
-  dependencies_json <- shiny_prerendered_extract_context(html_lines, "dependencies")
-  dependencies <- jsonlite::unserializeJSON(dependencies_json)
+  dependencies <- shiny_prerendered_extract_context_serialized(html_lines, "dependencies")
 
   # resolve package paths (this will happen automatically for the
   # development version of htmltools but this isn't on CRAN yet)
@@ -131,14 +173,41 @@ shiny_prerendered_html <- function(input_rmd, encoding, render_args) {
     dependencies <- append(dependencies, list(html_dependency_rsiframe()))
 
   # return html w/ dependencies
-  shinyHTML_with_deps(rendered_html, dependencies)
+  html_with_deps <- shinyHTML_with_deps(rendered_html, dependencies)
+
+  # The html template used to render the UI should contain the placeholder
+  # expected by shiny in `shiny:::renderPage()` which uses `htmltools::renderDocument`.
+  # This should be included during render() as a header include file. As a safety measure,
+  # if it is not present in the template, we add this placeholder at the end of
+  # the <head> element. This should not happen really.
+  # Context: https://github.com/rstudio/rmarkdown/pull/2249
+  if (!any(grepl(headContent <- "<!-- HEAD_CONTENT -->", html_with_deps, fixed = TRUE))) {
+    html_with_deps <- sub(
+      '</head>',
+      paste0('\n', headContent, '\n</head>'),
+      html_with_deps,
+      fixed = TRUE,
+      useBytes = TRUE)
+    Encoding(html_with_deps) <- "UTF-8"
+  }
+  html_with_deps
+}
+
+shiny_prerendered_ui <- function(html, deps) {
+  # prerendered html is a full document that should not be expanded in shiny::renderPage()
+  # so make shiny aware of that with the attributes 'html_document' to mimic the result of
+  # htmltools::htmlTemplate(document_ = TRUE).
+  # https://github.com/rstudio/rmarkdown/issues/1912
+  html_doc <- htmltools::tagList(html)
+  class(html_doc) <- c("html_document", class(html_doc))
+  # attach dependencies to final html
+  htmltools::attachDependencies(html_doc, deps)
 }
 
 shiny_prerendered_prerender <- function(
   input_rmd,
   rendered_html,
   output_dir,
-  encoding,
   prerender_option
 ) {
   if (file.access(output_dir, 2) != 0) {
@@ -169,7 +238,7 @@ shiny_prerendered_prerender <- function(
   }
 
   # find external resources referenced by the file
-  external_resources <- find_external_resources(input_rmd, encoding)
+  external_resources <- find_external_resources(input_rmd)
 
   # get paths to external resources
   input_files <- c(input_rmd, file.path(output_dir, external_resources$path))
@@ -185,18 +254,29 @@ shiny_prerendered_prerender <- function(
   html_lines <- read_utf8(rendered_html)
 
   # check that all html dependencies exist
-  dependencies_json <- shiny_prerendered_extract_context(html_lines, "dependencies")
-  dependencies <- jsonlite::unserializeJSON(dependencies_json)
+  dependencies <- tryCatch(
+    shiny_prerendered_extract_context_serialized(html_lines, "dependencies"),
+    error = function(...) NULL
+  )
+  if (is.null(dependencies)) {
+    # Pre-render needed: failed to parse deps from pre-rendered HTML
+    return(TRUE)
+  }
 
   pkgsSeen <- list()
   for (dep in dependencies) {
     if (is.null(dep$package)) {
-      # if the file doesn't exist at all, render again
-      if (!file.exists(dep$src$file)) {
-        # might create a missing file compile-time error,
-        #   but that's better than a missing file prerendered error
-        return(TRUE)
+      src_file <- dep$src$file
+      if (!is.null(src_file)) {
+        if (!file.exists(src_file)) {
+          # might create a missing file compile-time error,
+          #   but that's better than a missing file prerendered error
+          return(TRUE)
+        }
       }
+      # if there is a dep$src$href but no dep$package,
+      # then we can't determine where the file came from.
+      # Ignore checking for the href files for now, as pkg versions are checked below
     } else {
       depPkg <- dep$package
       depVer <- dep$pkgVersion
@@ -207,7 +287,7 @@ shiny_prerendered_prerender <- function(
         #   means old prerender version, render again
         if (!isTRUE(get_package_version_string(depPkg) == depVer)) {
           # was not rendered with the same R package. must render again
-          return (TRUE)
+          return(TRUE)
         }
         pkgsSeen[[depPkg]] <- depVer
       }
@@ -216,9 +296,15 @@ shiny_prerendered_prerender <- function(
   # all html dependencies are accounted for
 
   # check for execution package version differences
-  execution_json <- shiny_prerendered_extract_context(html_lines, "execution_dependencies")
-  execution_info <- jsonlite::unserializeJSON(execution_json)
-  execution_pkg_names <- execution_info$packages$package
+  execution_info <- tryCatch(
+    shiny_prerendered_extract_context_serialized(html_lines, "execution_dependencies"),
+    error = function(...) NULL
+  )
+  if (is.null(execution_info)) {
+    # Pre-render needed: failed to parse execution deps from pre-rendered HTML
+    return(TRUE)
+  }
+  execution_pkg_names <- execution_info$packages$packages
   execution_pkg_versions <- execution_info$packages$version
   for (i in seq_along(execution_pkg_names)) {
     if (!identical(
@@ -252,7 +338,7 @@ shiny_prerendered_append_dependencies <- function(input, # always UTF-8
     if (is.null(dependency$package) && is.character(dependency$src$file)) {
 
       # check for a package directory parent
-      package_dir <- package_root(dependency$src$file)
+      package_dir <- proj_root(dependency$src$file)
       # if we have one then populate the package field and make the
       # src$file relative to the package
       if (!is.null(package_dir)) {
@@ -273,6 +359,10 @@ shiny_prerendered_append_dependencies <- function(input, # always UTF-8
       dependency$src = list(href = unname(dependency$src))
     }
 
+    if (!is.null(dependency$package)) {
+      dependency$pkgVersion <- get_package_version_string(dependency$package)
+    }
+
     # return dependency
     dependency
   })
@@ -280,10 +370,14 @@ shiny_prerendered_append_dependencies <- function(input, # always UTF-8
   # remove NULLs (excluded dependencies)
   dependencies <- dependencies[!sapply(dependencies, is.null)]
 
-  # append them to the file (guarnateed to be UTF-8)
+  # append them to the file (guaranteed to be UTF-8)
   con <- file(input, open = "at", encoding = "UTF-8")
   on.exit(close(con), add = TRUE)
 
+  # Add newline before adding any raw html dependency script
+  # https://github.com/rstudio/rmarkdown/issues/2336
+  writeLines("", con = con)
+
   # write deps to connection
   dependencies_json <- jsonlite::serializeJSON(dependencies, pretty = FALSE)
   shiny_prerendered_append_context(con, "dependencies", dependencies_json)
@@ -297,6 +391,16 @@ shiny_prerendered_append_dependencies <- function(input, # always UTF-8
   shiny_prerendered_append_context(con, "execution_dependencies", execution_json)
 }
 
+shiny_prerendered_extract_context_serialized <- function(html_lines, context = "dependencies") {
+  json_str <- shiny_prerendered_extract_context(html_lines, context)
+  json_str <- unique(json_str)
+  if (length(json_str) > 1) {
+    warning("Multiple ", context, " contexts found in prerendered HTML, using last.")
+    json_str <- json_str[length(json_str)]
+  }
+  jsonlite::unserializeJSON(json_str)
+}
+
 
 #' Clean prerendered content for the specified Rmd input file
 #'
@@ -344,10 +448,10 @@ shiny_prerendered_clean <- function(input) {
 shiny_prerendered_chunk <- function(context, code, singleton = FALSE) {
 
   # verify we are in runtime: shiny_prerendered
-  if (!identical(knitr::opts_knit$get("rmarkdown.runtime"),"shiny_prerendered"))
-      stop("The shiny_prerendered_chunk function can only be called from ",
-           "within runtime: shiny_prerendered",
-           call. = FALSE)
+  if (!is_shiny_prerendered(knitr::opts_knit$get("rmarkdown.runtime")))
+      stop2("The shiny_prerendered_chunk function can only be called from ",
+           "within a shiny server compatible document"
+      )
 
   # add the prerendered chunk to knit_meta
   knitr::knit_meta_add(list(
@@ -382,7 +486,7 @@ shiny_prerendered_server_start_code <- function(server_envir) {
 # We'll use this later to determine which .RData files in the _data
 # directory should actually be loaded (as some could be from chunks
 # that used to be cached / were cached under different names)
-shiny_prerendered_option_hook <- function(input, encoding) {
+shiny_prerendered_option_hook <- function(input) {
   function(options) {
 
     # convert chunk labels to contexts as necessary
@@ -392,13 +496,19 @@ shiny_prerendered_option_hook <- function(input, encoding) {
     if (identical(options$context, "data")) {
       data_file <- shiny_prerendered_data_file_name(options$label,
                                                     options$cache > 0)
-      data_file <- to_utf8(data_file, encoding)
       data_dir <- shiny_prerendered_data_dir(input, create = TRUE)
       index_file <- shiny_prerendered_data_chunks_index(data_dir)
       conn <- file(index_file, open = "ab", encoding = "UTF-8")
       on.exit(close(conn), add = TRUE)
       write(data_file, file = conn, append = TRUE)
     }
+
+    if (identical(options$context, "server")) {
+      # if empty server context, set a default server function
+      if (all(xfun::is_blank(options$code))) {
+        options$code <- "# empty server context"
+      }
+    }
     options
   }
 }
@@ -465,7 +575,7 @@ shiny_prerendered_evaluate_hook <- function(input) {
 
     # otherwise parse so we can throw an error for invalid code
     else {
-      parse(text = code)
+      xfun::parse_only(code)
       list()
     }
   }
@@ -548,7 +658,7 @@ shiny_prerendered_remove_contexts <- function(html_lines, contexts) {
 
 # Gather shiny_prerendred contexts and append them as script tags to
 # the passed file
-shiny_prerendered_append_contexts <- function(runtime, file, encoding) {
+shiny_prerendered_append_contexts <- function(runtime, file) {
 
   # collect contexts
   shiny_prerendered_contexts <- knit_meta_reset(class = "shiny_prerendered")
@@ -556,12 +666,11 @@ shiny_prerendered_append_contexts <- function(runtime, file, encoding) {
 
     # validate we are in runtime: shiny_prerendered
     if (!is_shiny_prerendered(runtime)) {
-      stop("The code within this document requires runtime: shiny_prerendered",
-           call. = FALSE)
+      stop2("The code within this document requires server: shiny")
     }
 
     # open the file
-    con <- file(file, open = "at", encoding = encoding)
+    con <- file(file, open = "at", encoding = 'UTF-8')
     on.exit(close(con), add = TRUE)
 
     # track singletons
@@ -570,7 +679,7 @@ shiny_prerendered_append_contexts <- function(runtime, file, encoding) {
     # append the contexts as script tags
     for (context in shiny_prerendered_contexts) {
 
-      # resovle singletons
+      # resolve singletons
       if (isTRUE(context$singleton)) {
         found_singleton <- FALSE
         for (singleton in singletons) {
@@ -603,7 +712,7 @@ shiny_prerendered_append_context <- function(con, name, code) {
 
 # Prerendred data_dir for a given Rmd input file
 shiny_prerendered_data_dir <- function(input, create = FALSE) {
-  data_dir <- paste0(tools::file_path_sans_ext(input), "_data")
+  data_dir <- paste0(xfun::sans_ext(input), "_data")
   if (create && !dir_exists(data_dir))
     dir.create(data_dir)
   data_dir
@@ -622,7 +731,7 @@ shiny_prerendered_data_dir <- function(input, create = FALSE) {
 #     ensures that we only read .RData for chunks that were
 #     included in the last rendered document.
 #
-# (2) We want to load data into the server environemnt in the
+# (2) We want to load data into the server environment in the
 #     exact same chunk order that it appears in the document.
 #
 shiny_prerendered_data_load <- function(input_rmd, server_envir) {
@@ -653,3 +762,18 @@ shiny_prerendered_data_file_name <- function(label, cache) {
   type <- ifelse(cache, ".cached", "")
   sprintf("%s%s.RData", label, type)
 }
+
+# Use me instead of html_dependency_bootstrap() in a shiny runtime to get
+# dynamic theming (i.e., have it work with session$setCurrentTheme())
+shiny_bootstrap_lib <- function(theme) {
+  theme <- resolve_theme(theme)
+  if (!is_bs_theme(theme)) {
+    return(NULL)
+  }
+  if (!is_available("shiny", "1.6.0")) {
+    stop(
+      "Using a {bslib} theme with `runtime: shiny` requires shiny 1.6.0 or higher."
+    )
+  }
+  shiny::bootstrapLib(theme)
+}
